unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ConsoleApp, CoolTrayIcon, ImgList, Grids, SortGrid, ExtCtrls, Pinggg,
  ComCtrls, IniFiles;

type
  TfrmMain = class(TForm)
    icon: TCoolTrayIcon;
    il: TImageList;
    timer: TTimer;
    GroupBox1: TGroupBox;
    sgNet: TSortGrid;
    GroupBox2: TGroupBox;
    cbUrls: TComboBox;
    sb: TStatusBar;
    cbTimeout: TCheckBox;
    procedure iconClick(Sender: TObject);
    procedure timerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgNetDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormDestroy(Sender: TObject);
    procedure cbUrlsChange(Sender: TObject);
    procedure cbTimeoutClick(Sender: TObject);
  private
    { Private declarations }
    list: TStringList;
    urls: TStringList;
    urlIndex: integer;
    pingRetries: integer;
    pingLatency: integer;
    pingTimer: integer;
    procedure LoadPingLog(filename: string);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

const pingLog_prefix = 'ping(';
const pingLog_postfix = ').log';
const timeOut = 60000;

procedure TfrmMain.FormCreate(Sender: TObject);
var i:integer; ini: TIniFile;
begin
  // loading values from ini file
  Ini := TIniFile.Create( Copy( Paramstr(0), 1, Length(Paramstr(0))-3) + 'ini');
  try
    pingRetries := StrToInt(Ini.ReadString('Ping', 'Retries', '3'));
    pingLatency := StrToInt(Ini.ReadString('Ping', 'Latency', '1'));
    pingTimer := StrToInt(Ini.ReadString('Ping', 'Timer', '60'));
  finally
    Ini.Free;
  end;

  with sgNet do begin
    Cells[0,0] := 'Timestamp';  ColWidths[0] := 120;
    Cells[1,0] := 'Ping (ms)';  ColWidths[1] := 60;
    Cells[2,0] := 'Range (0-5000 ms)';    ColWidths[2] := 120;
  end;

  urls := TStringList.Create();
  urlIndex := 0;

  if FileExists('Pingator.url') then
    urls.LoadFromFile('Pingator.url')
  else begin
    Application.MessageBox('File "pingator.url" which should list URL addresses to ping was not found!', 'File not found', MB_ICONSTOP);
    Application.Terminate();
    exit;
  end;

  for i := 1 to urls.Count do
    cbUrls.Items.Add(urls[i-1]);

  cbUrls.ItemIndex := 0;
  cbUrlsChange(self);

  timer.Interval := pingTimer * 1000;
  timer.Enabled := true;
end;

procedure TfrmMain.iconClick(Sender: TObject);
begin
  icon.ShowMainForm();
end;

procedure TfrmMain.timerTimer(Sender: TObject);
var
  i,j,k: integer;
  ticks: DWORD;
  total: DWORD;
  avg: integer;
  failures: integer;
  ts: string;
  pg: string;
  url: string;
  F: TextFile;
  filename: string;
  handle: integer;
begin
  for k := 0 to urls.Count - 1 do begin
    url := urls[k];

    total := 0;
    failures := 0;
    for i := 1 to pingRetries do begin
      sb.Panels[0].Text := 'Pinging at ' + url + '... (try ' + IntToStr(i) + '/' + IntToStr(pingRetries) + ')';
      Application.ProcessMessages();
      if not Ping(url, @ticks) then begin
        Inc(failures);
      end else begin
        total := total + ticks;
      end;
      sb.Panels[0].Text := sb.Panels[0].Text + ' [waiting ' + IntToStr(pingLatency) + ' sec(s)]';
      for j := 1 to 10 * pingLatency do begin
        Sleep(100);
        Application.ProcessMessages();
      end;
    end;

    sb.Panels[0].Text := '';
    Application.ProcessMessages();

    ts := DateTimeToStr(Now());
    if failures >= pingRetries then
      pg := IntToStr(timeOut) else
      pg := IntToStr(total div (pingRetries - failures));

    filename := pingLog_prefix + urls[k] + pingLog_postfix;
    if not FileExists(filename) then begin
      handle := FileCreate(filename);
      FileClose(handle);
    end;
    AssignFile(F, filename);
    Append(F);
    Writeln(F, ts + '=' + pg);
    CloseFile(F);

    if k = urlIndex then begin
      if cbTimeout.Checked and (StrToInt(pg) < timeOut) then begin end else begin
        sgNet.Cells[0,sgNet.RowCount] := ts;
        sgNet.Cells[1,sgNet.RowCount] := pg;
        sgNet.RowCount := sgNet.RowCount + 1;
        sgNet.Row := sgNet.RowCount - 1;
      end;
    end;

  end;
end;

procedure TfrmMain.LoadPingLog(filename: string);
var i,p,t,d:integer; s:string;
begin
  list := TStringList.Create;
  filename := pingLog_prefix + filename + pingLog_postfix;
  if FileExists(filename) then
    list.LoadFromFile(filename);

  sgNet.RowCount := 2;
  sgNet.Cells[0,1] := '';
  sgNet.Cells[1,1] := '';
  sgNet.Cells[2,1] := '';
  t := 1;
  for i := 1 to list.Count do begin
    s := list[i-1];
    p := Pos('=', s);
    if p > 0 then begin
      try
        d := StrToInt(Copy(s,p+1,Length(s)-p));
        if cbTimeout.Checked and (d < timeOut) then continue;
        sgNet.Cells[0,t] := Copy(s,0,p-1);
        sgNet.Cells[1,t] := Copy(s,p+1,Length(s)-p);
        t := t + 1;
      except
      end;
    end;
  end;
  if t > 1 then sgNet.RowCount := t;
  sgNet.Row := sgNet.RowCount - 1;
  list.Destroy;
end;


procedure TfrmMain.sgNetDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var r: TRect; b: integer;
begin
  if (ACol = 2) and (ARow > 0) and (sgNet.Cells[1,ARow] <> '') then begin
    try
      b := StrToInt(sgNet.Cells[1,ARow]) div 50;
      if b > 100 then b := 100;
    except
      b := 0;
    end;

    r.Left := Rect.Left + 1;
    r.Top := Rect.Top + 1;
    r.Bottom := Rect.Bottom - 1;
    r.Right := r.Left + 3 + b;

    if b = 100 then
      sgNet.Canvas.Brush.Color := clRed else
      sgNet.Canvas.Brush.Color := clLime;

    sgNet.Canvas.FillRect(r);
    sgNet.Canvas.Brush.Color := clBlack;
    sgNet.Canvas.FrameRect(r);
  end;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  icon.IconVisible := false;
end;

procedure TfrmMain.cbUrlsChange(Sender: TObject);
begin
  urlIndex := cbUrls.ItemIndex;
  LoadPingLog(urls[urlIndex]);
end;

procedure TfrmMain.cbTimeoutClick(Sender: TObject);
begin
  cbUrlsChange(self);
end;

end.
